# coding: utf-8

import os
import re

from recycle.config import TRASH_PATH, TRASH_REGEX
from recycle.lib import (
    my_print,
    operations,
    directory_exists,
    search_files,
    replace_file,
    execute_move,
    remove_empty_dir,
)


def recover_by_id(file_id, recover_path, absolute_dir):
    if replace_file(recover_path):
        absolute_file = os.path.join(absolute_dir, file_id)
        execute_move(absolute_file, recover_path)
        return remove_empty_dir(absolute_dir)


def recover_by_regrex(absolute_dir, file_regex, recover_path, reverse):
    for file_name_dir in search_files(absolute_dir, file_regex):
        recover_trash_file_dir = os.path.join(recover_path, file_name_dir)
        absolute_trash_file_dir = os.path.join(absolute_dir, file_name_dir)
        if not directory_exists(absolute_trash_file_dir):
            return
        trash_files_list = search_files(absolute_trash_file_dir, TRASH_REGEX)
        for file_name in sorted(trash_files_list, reverse=reverse):
            recover_by_id(file_name, recover_trash_file_dir, absolute_trash_file_dir)
            break
        remove_empty_dir(absolute_trash_file_dir)


def recover_from_trash(trash_dir, file_regex, reverse):
    if trash_dir.startswith(TRASH_PATH):
        relative_dir = trash_dir[len(TRASH_PATH) :]
    relative_dir = trash_dir.strip("/")
    absolute_dir = os.path.join(TRASH_PATH, relative_dir)
    recover_path = "/" + relative_dir

    if not directory_exists(absolute_dir):
        return

    if not os.path.isdir(recover_path):
        os.makedirs(recover_path, exist_ok=True)

    if re.match(TRASH_REGEX, file_regex):
        current_dir = os.getcwd()
        recover_by_id(file_regex, recover_path, absolute_dir)
        if trash_dir == current_dir:
            my_print("\nPlease run: \n\n\tcd ..;cd -")
        return
    recover_by_regrex(absolute_dir, file_regex, recover_path, reverse)


def main():
    for parent_dir, file_regex, reverse in operations():
        recover_from_trash(parent_dir, file_regex, reverse)



# coding: utf-8

import json
import os

HOME = os.path.expanduser("~")
INIT_TRASH_PATH = HOME + "/.Trash"
TRASH_DATETIME_FORMAT = "%Y-%m-%d_%H:%M:%S%f_"
TRASH_REGEX = "\d{4}-\d{2}-\d{2}_\d{2}:\d{2}:\d{8}_[0-9\.]+[A-Z]?"
CONFIG_PATH = HOME + "/.py_recycle.json"
if not os.path.isfile(CONFIG_PATH):
    open(CONFIG_PATH, "w").write("{}")
config = json.loads(open(CONFIG_PATH).read())
EMOJIS = {"file": "📄", "directory": "📁"}
COLORS = {
    "white": 0,
    "black": 30,
    "red": 31,
    "green": 32,
    "orange": 33,
    "blue": 34,
    "magenta": 35,
    "cyan": 36,
    "light_gray": 37,
}

FILE_SIZE_COLORS = {
    "file size unit": [
        "file size < 10unit",
        "file size < 100unit",
        "file size < 1000unit",
        "file size < 10000unit",
    ],
    "choice of color": list(COLORS.keys()) + list(map(lambda x: "blod " + x, COLORS)),
    "b": ["white"] * 4,
    "B": ["white"] * 4,
    "K": ["white"] * 3 + ["green"],
    "M": ["green", "green", "blod green", "blue"],
    "G": ["blue", "blue", "blod blue", "orange"],
    "T": ["orange", "orange", "blood orange", "red"],
    "P": ["red", "red", "blood red", "blood red"],
}
# User Config
ENABLE_EMOJI = config.get("ENABLE_EMOJI", True)
ENABLE_COLOR = config.get("ENABLE_COLOR", True)
TREE_ALL_DIRECTORY_SIZE = config.get("TREE_ALL_DIRECTORY_SIZE", True)
TRASH_PATH = config.get("TRASH_PATH") or INIT_TRASH_PATH
FILE_SIZE_COLORS = config.get("FILE_SIZE_COLORS") or FILE_SIZE_COLORS



# coding: utf-8

import os
import sys
import re
import datetime
import subprocess

from recycle.config import (
    TRASH_PATH,
    TRASH_DATETIME_FORMAT,
    FILE_SIZE_COLORS,
    ENABLE_COLOR,
    COLORS,
)


def get_current_path():
    try:
        return os.path.abspath(os.getcwd())
    except Exception:
        return None


def get_colorful_str(s, end="\n", color_code=None):
    if color_code and ENABLE_COLOR:
        if " " in color_code:
            font_weight, color_code = 1, color_code.split(" ")[1]
        else:
            font_weight, color_code = 0, color_code
        color_code = COLORS[color_code]
        s = "\033[{}m\033[1;{}m{}\033\033[0m".format(font_weight, color_code, s)
    return "{}{}".format(s, end)


def my_print(s, end="\n", color_code=None):
    sys.stdout.write(get_colorful_str(s, end, color_code))


def get_size_color_code(size_str):
    index = len(size_str[:-1].split(".")[0]) - 1
    sign = size_str[-1]
    return FILE_SIZE_COLORS.get(sign, ["white"] * 4)[index]


def get_path_size_str(file_path):
    return subprocess.check_output(["du", "-sh", file_path]).split()[0].decode("utf-8")


def generate_trash_file_name(file_path):
    return datetime.datetime.now().strftime(TRASH_DATETIME_FORMAT) + get_path_size_str(
        file_path
    )


def directory_exists(directory):
    if os.path.isdir(directory):
        return True
    my_print('Directory "{}" not exists.'.format(directory))
    return False


def search_files(directory, file_regex):
    if file_regex == "\.":
        return [directory]
    if not (file_regex and directory_exists(directory)):
        return []

    files_list = os.listdir(directory)
    if file_regex in files_list:
        return [file_regex]
    file_compile = re.compile(file_regex)
    return list(filter(file_compile.match, files_list))


def execute_delete(path):
    my_print("rm -rf {}".format(path))
    os.system("rm -rf {}".format(re.escape(path)))


def execute_move(source, destination):
    my_print("mv {} {}".format(source, destination))
    source = re.escape(source)
    destination = re.escape(destination)
    # Will change current directory to source
    os.system("mv {} {}".format(source, destination))


def remove_empty_dir(absolute_path):
    while not os.path.exists(absolute_path):
        absolute_path = absolute_path[: absolute_path.rindex("/")]
        if absolute_path == TRASH_PATH:
            break

    while os.path.isdir(absolute_path) and not os.listdir(absolute_path):
        os.rmdir(absolute_path)
        absolute_path = absolute_path[: absolute_path.rindex("/")]
        if absolute_path == TRASH_PATH:
            break


def my_input(s):
    try:
        return raw_input(s)
    except:
        return input(s)


def input_yes(s):
    return my_input(s).lower() in ["yes", "y"]


def replace_file(file_path):
    if os.path.exists(file_path):
        if input_yes("{} already exists replace it? [N/y]".format(file_path)):
            trash_file = os.path.join(
                TRASH_PATH + file_path, generate_trash_file_name(file_path)
            )
            execute_move(file_path, trash_file)
            return True
    else:
        return True


def get_absolute_dirs(dirs):
    absolute_dirs = []

    for i in dirs:
        if i == "..":
            absolute_dirs.pop(-1)
        elif absolute_dirs and not (i or absolute_dirs[-1]):
            pass
        else:
            absolute_dirs.append(i)
    return absolute_dirs


def get_parent_dir_and_file_regex(input_arg):
    reverse = True
    if input_arg == "/":
        return "/", "\.", reverse
    parent_dir = get_current_path()
    dirs = input_arg.split("/")

    if dirs:
        if not dirs[0]:
            # use absolute path
            dirs = get_absolute_dirs(dirs)
        else:
            dirs = get_absolute_dirs(parent_dir.split("/") + dirs)
        parent_dir = "/".join(dirs[:-1]) or "/"

    file_regex = dirs[-1] if dirs else input_arg
    return parent_dir, file_regex, reverse


def operations():
    for arg in sys.argv[1:]:
        if arg != "/":
            arg = arg.rstrip("/")
            arg = arg[2:] if arg[:2] == "./" else arg

        parent_dir, file_regex, reverse = get_parent_dir_and_file_regex(arg)
        if not arg:
            continue
        if file_regex == "*":
            file_regex = ".*"
        yield parent_dir, file_regex, reverse



# coding: utf-8

import os
import re
import sys

from recycle.config import (
    TRASH_PATH,
    TRASH_REGEX,
    ENABLE_EMOJI,
    EMOJIS,
    TREE_ALL_DIRECTORY_SIZE,
)

from recycle.lib import (
    my_print,
    directory_exists,
    operations,
    search_files,
    get_current_path,
    get_path_size_str,
    get_size_color_code,
)


def print_file(file_name):
    if ENABLE_EMOJI:
        my_print(EMOJIS["file"], end=" ")
    color_code = get_size_color_code(file_name.split("_")[-1])
    my_print(file_name, color_code=color_code)


def print_directory(directory_name, directory_path, is_trash=False, is_header=False):
    if is_trash:
        if ENABLE_EMOJI:
            my_print(EMOJIS["directory"], end=" ")
        color_code = get_size_color_code(directory_name.split("_")[-1])
        my_print(directory_name, color_code=color_code)

    elif is_header:
        size_str = get_path_size_str(directory_path)
        my_print(
            "{} [{}]".format(directory_name, size_str),
            color_code=get_size_color_code(size_str),
        )
    elif TREE_ALL_DIRECTORY_SIZE:
        size_str = get_path_size_str(directory_path)
        my_print(
            "{} [{}]".format(directory_name, size_str),
            color_code=get_size_color_code(size_str),
        )
    else:
        my_print(directory_name)


def print_branch(file_name, file_path, is_last_one, parent_str, stop_regex):
    stop_tree = re.match(stop_regex, file_name) if stop_regex else False
    tmp_str = parent_str

    if is_last_one:
        header = "└── "
        parent_str += "    "
    else:
        header = "├── "
        parent_str += "│   "

    sys.stdout.write(tmp_str + header)

    if not os.path.isdir(file_path):
        print_file(file_name)
        return

    if not stop_tree:
        print_directory(file_name, file_path)
        return print_tree(file_path, parent_str=parent_str, stop_regex=stop_regex)

    print_directory(file_name, file_path, True)


def print_tree(directory, stop_regex=None, parent_str="", first_print_regex=True):
    if first_print_regex and stop_regex:
        files = sorted(
            os.listdir(directory),
            key=lambda x: x if re.match(stop_regex, x) else "X" + x,
        )
    else:
        files = sorted(os.listdir(directory))

    files_number = len(files)
    for i in range(files_number):
        file_name = files[i]
        file_path = os.path.join(directory, file_name)
        is_last_one = i == files_number - 1
        print_branch(file_name, file_path, is_last_one, parent_str, stop_regex)


def main():
    tree_pwd = True
    for parent_dir, file_regex, _ in operations():
        tree_pwd = False
        if not parent_dir.startswith(TRASH_PATH):
            parent_dir = TRASH_PATH + parent_dir
        for file_name in search_files(parent_dir, file_regex):
            file_path = os.path.join(parent_dir, file_name)
            if os.path.exists(file_path):
                print_directory(file_path, file_path, is_header=True)
                if os.path.isdir(file_path):
                    print_tree(file_path, stop_regex=TRASH_REGEX)
                my_print("\n")
    if tree_pwd:
        directory = get_current_path()
        if not directory.startswith(TRASH_PATH):
            directory = TRASH_PATH + directory

        if directory_exists(directory):
            print_directory(directory, directory, is_header=True)
            print_tree(directory, stop_regex=TRASH_REGEX)



# coding: utf-8

import os
import re

from recycle.config import TRASH_PATH, TRASH_REGEX
from recycle.lib import (
    operations,
    directory_exists,
    search_files,
    execute_delete,
    remove_empty_dir,
    input_yes,
    get_colorful_str,
)


def delete_by_id(absolute_file):
    return execute_delete(absolute_file)


def delete_by_regrex(absolute_dir, file_regex, reverse):
    for file_name_dir in search_files(absolute_dir, file_regex):
        absolute_trash_file_dir = os.path.join(absolute_dir, file_name_dir)
        if not directory_exists(absolute_trash_file_dir):
            return
        execute_delete(absolute_trash_file_dir)
        remove_empty_dir(absolute_trash_file_dir)


def permanently_delete(trash_dir, file_regex, reverse):
    if trash_dir.startswith(TRASH_PATH):
        relative_dir = trash_dir[len(TRASH_PATH) :].strip("/")
    else:
        relative_dir = trash_dir.strip("/")
    absolute_dir = os.path.join(TRASH_PATH, relative_dir)
    if not directory_exists(absolute_dir):
        return
    absolute_file = os.path.join(absolute_dir, file_regex)
    if not input_yes(
        "\tDelete {} ? [N/y]".format(
            get_colorful_str(
                absolute_file.replace("\.", ""), color_code="bold red", end=""
            )
        )
    ):
        return
    if re.match(TRASH_REGEX, file_regex):
        return delete_by_id(absolute_file)
    delete_by_regrex(absolute_dir, file_regex, reverse)
    return remove_empty_dir(absolute_dir)


def main():
    for parent_dir, file_regex, reverse in operations():
        permanently_delete(parent_dir, file_regex, reverse)
